@@ -45,6 +45,8 @@
 #include "gedit2.h"
 #include "gedit-plugin-manager.h"
 
+#include "gedit-encodings-dialog.h"
+
 #include "gnome-print-font-picker.h"
 
 /* To be syncronized with gedit-preferences.glade2 */
@@ -58,6 +60,7 @@
 #define LINE_NUMBERS_SETTINGS	7
 #define PRINT_FONTS_SETTINGS	8
 #define PLUGIN_MANAGER_SETTINGS 9
+#define LOAD_SETTINGS		10
 
 enum
 {
@@ -66,6 +69,14 @@ enum
 	NUM_COLUMNS
 };
 
+enum
+{
+	COLUMN_ENCODING_NAME = 0,
+	COLUMN_ENCODING_POINTER,
+	COLUMN_INDEX,
+	ENCODING_NUM_COLS
+};
+
 struct _GeditPreferencesDialogPrivate
 {
 	GtkWidget* categories_tree;
@@ -126,6 +137,13 @@ struct _GeditPreferencesDialogPrivate
 
 	/* Plugin/Manager */
 	GtkWidget	*plugin_manager;
+
+	/* Load page */
+	GtkWidget	*encodings_treeview;
+	GtkWidget	*add_enc_button;
+	GtkWidget	*remove_enc_button;
+	GtkWidget	*up_enc_button;
+	GtkWidget	*down_enc_button;
 	
 };
 
@@ -198,6 +216,7 @@ static void gedit_preferences_dialog_display_line_numbers_checkbutton_toggled (G
 									       GeditPreferencesDialog *dlg);
 static gboolean gedit_preferences_dialog_setup_plugin_manager_page (GeditPreferencesDialog *dlg, 
 								    GladeXML *gui);
+static gboolean gedit_preferences_dialog_setup_load_page (GeditPreferencesDialog *dlg, GladeXML *gui);
 
 static gint last_selected_page_num = FONT_COLORS_SETTINGS;
 static GtkDialogClass* parent_class = NULL;
@@ -210,6 +229,7 @@ static CategoriesTreeItem editor_behavior [] =
 	{N_("Wrap mode"), NULL, WRAP_MODE_SETTINGS},
 	{N_("Line numbers"), NULL , LINE_NUMBERS_SETTINGS},
 	
+	{N_("Load"), NULL, LOAD_SETTINGS },
  	{N_("Save"), NULL, SAVE_SETTINGS },
 	{N_("Undo"), NULL, UNDO_SETTINGS},
 
@@ -299,11 +319,17 @@ gedit_preferences_dialog_init (GeditPreferencesDialog *dlg)
 
 	gedit_preferences_dialog_add_buttons (dlg);	
 	
-	hbox = gtk_hbox_new (FALSE, 12);
+	hbox = gtk_hbox_new (FALSE, 18);
 	
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
-	
-	r = gtk_vbox_new (FALSE, 0);
+	/*
+	gtk_container_set_border_width (GTK_CONTAINER (dlg), 12);
+	*/
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 10);
+	/*
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dlg)->vbox), 12);
+	*/
+
+	r = gtk_vbox_new (FALSE, 6);
 	
 	label = gtk_label_new_with_mnemonic (_("Cat_egories:"));
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
@@ -311,7 +337,7 @@ gedit_preferences_dialog_init (GeditPreferencesDialog *dlg)
 
 	ct = gedit_preferences_dialog_create_categories_tree (dlg);
 		
-	gtk_box_pack_start (GTK_BOX (r), label, FALSE, FALSE, 6);
+	gtk_box_pack_start (GTK_BOX (r), label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (r), ct, TRUE, TRUE, 0);
 
 	l = gedit_preferences_dialog_create_notebook (dlg);
@@ -623,6 +649,7 @@ gedit_preferences_dialog_create_notebook (GeditPreferencesDialog *dlg)
 	gedit_preferences_dialog_setup_line_numbers_page (dlg, gui);
 	gedit_preferences_dialog_setup_print_fonts_page (dlg, gui);
 	gedit_preferences_dialog_setup_plugin_manager_page (dlg, gui);
+	gedit_preferences_dialog_setup_load_page (dlg, gui);
 
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (dlg->priv->notebook), LOGO);
 	
@@ -1584,32 +1611,34 @@ gedit_preferences_dialog_print_font_restore_default_button_clicked (
 	g_return_if_fail (dlg->priv->headers_fontpicker != NULL);
 	g_return_if_fail (dlg->priv->numbers_fontpicker != NULL);
 
-	/* FIXME: define constants, or take them from schemas - Paolo */
 	if (gedit_prefs_manager_print_font_body_can_set ())
 	{
 		gnome_print_font_picker_set_font_name (
 				GNOME_PRINT_FONT_PICKER (dlg->priv->body_fontpicker),
-				_("Courier 9"));
+				gedit_prefs_manager_get_default_print_font_body ());
 
-		gedit_prefs_manager_set_print_font_body (_("Courier 9"));
+		gedit_prefs_manager_set_print_font_body (
+				gedit_prefs_manager_get_default_print_font_body ());
 	}
 	
 	if (gedit_prefs_manager_print_font_header_can_set ())
 	{
 		gnome_print_font_picker_set_font_name (
 				GNOME_PRINT_FONT_PICKER (dlg->priv->headers_fontpicker),
-				_("Helvetica 11"));
+				gedit_prefs_manager_get_default_print_font_header ());
 		
-		gedit_prefs_manager_set_print_font_header (_("Helvetica 11"));
+		gedit_prefs_manager_set_print_font_header (
+				gedit_prefs_manager_get_default_print_font_header ());
 	}
 		
 	if (gedit_prefs_manager_print_font_numbers_can_set ())
 	{
 		gnome_print_font_picker_set_font_name (
 				GNOME_PRINT_FONT_PICKER (dlg->priv->numbers_fontpicker),
-				_("Helvetica 8"));
+				gedit_prefs_manager_get_default_print_font_numbers ());
 		
-		gedit_prefs_manager_set_print_font_numbers (_("Helvetica 8"));
+		gedit_prefs_manager_set_print_font_numbers (
+				gedit_prefs_manager_get_default_print_font_numbers ());
 	}
 }
 
@@ -1859,3 +1888,407 @@ gedit_preferences_dialog_setup_plugin_manager_page (GeditPreferencesDialog *dlg,
 	
 	return TRUE;
 }
+
+static gboolean
+add_enc_to_list (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
+{
+	GSList **list;
+
+	GValue value = {0, };
+	const GeditEncoding *enc;
+
+	gedit_debug (DEBUG_PREFS, "");
+
+	list = (GSList **)data;
+
+	gtk_tree_model_get_value (model, iter, COLUMN_ENCODING_POINTER, &value);
+
+	enc = (const GeditEncoding *) g_value_get_pointer (&value);
+	g_return_val_if_fail (enc != NULL, TRUE);
+	
+	{
+		gchar *name;
+		name = gedit_encoding_to_string (enc);
+		/*
+		g_print ("Add %s to list\n", name);
+		*/
+		g_free (name);
+	}
+
+	*list = g_slist_prepend (*list, (gpointer) enc);
+
+	g_value_unset (&value);
+
+	return FALSE;
+};
+
+static void 
+update_encodings_list (GeditPreferencesDialog *dlg)
+{
+	GtkTreeModel *model;
+	GSList *enc_list = NULL;
+	
+	gedit_debug (DEBUG_PREFS, "");
+
+	g_return_if_fail (dlg != NULL);
+	g_return_if_fail (gedit_prefs_manager_encodings_can_set ());
+	
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (dlg->priv->encodings_treeview));
+	g_return_if_fail (model != NULL);
+
+	gtk_tree_model_foreach (model, add_enc_to_list, &enc_list);
+
+	enc_list = g_slist_reverse (enc_list);
+
+	gedit_prefs_manager_set_encodings (enc_list);
+
+	g_slist_free (enc_list);
+}
+
+static GtkTreeModel*
+create_encodings_treeview_model (void)
+{
+	GtkListStore *store;
+	GtkTreeIter iter;
+	GSList const *list;
+	gint i = 0;
+	
+	gedit_debug (DEBUG_PREFS, "");
+
+	/* create list store */
+	store = gtk_list_store_new (ENCODING_NUM_COLS, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_INT);
+
+	/* add data to the list store */
+	list = gedit_prefs_manager_get_encodings ();
+	
+	while (list != NULL)
+	{
+		const GeditEncoding* enc;
+		gchar *name;
+
+		enc = (const GeditEncoding*) list->data;
+		name = gedit_encoding_to_string (enc);
+		
+		gtk_list_store_append (store, &iter);
+		gtk_list_store_set (store, &iter,
+				    COLUMN_ENCODING_NAME, name,
+				    COLUMN_ENCODING_POINTER, enc,
+				    COLUMN_INDEX, i,
+				    -1);
+
+		g_free (name);
+		
+		list = g_slist_next (list);
+		++i;
+	}
+	
+	return GTK_TREE_MODEL (store);
+}
+
+static void
+gedit_preferences_dialog_add_enc_button_clicked (GtkButton *button, GeditPreferencesDialog *dlg)
+{
+	gedit_debug (DEBUG_PREFS, "");
+
+	g_return_if_fail (dlg != NULL);
+	
+	gedit_encodings_dialog_run (dlg);
+}
+
+static void
+gedit_preferences_dialog_remove_enc_button_clicked (GtkButton *button, GeditPreferencesDialog *dlg)
+{
+	GtkTreeSelection *selection;
+	GtkTreeIter iter;
+	GtkTreePath *path;
+
+	GtkTreeModel *model;
+
+	gedit_debug (DEBUG_PREFS, "");
+
+	g_return_if_fail (dlg != NULL);
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dlg->priv->encodings_treeview));
+	g_return_if_fail (selection != NULL);
+
+	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+		return;
+	
+	path = gtk_tree_model_get_path (model, &iter);
+
+	gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+
+	update_encodings_list (dlg);
+
+	gtk_tree_selection_select_path (selection, path);
+
+	/* If the last item is removed, select the prev item */
+	if (!gtk_tree_selection_get_selected (selection, NULL, NULL))
+	{
+		if (gtk_tree_path_prev (path))
+			gtk_tree_selection_select_path (selection, path);
+	}
+			
+	gtk_tree_path_free (path);
+}
+
+
+static void
+gedit_preferences_dialog_encodings_treeview_selection_changed (GtkTreeSelection *selection, GeditPreferencesDialog *dlg)
+{
+	gboolean selected;
+	GtkTreeModel *model;
+	gint n_rows;
+	GtkTreeIter iter;
+	gboolean disable_up = TRUE;
+	gboolean disable_down = TRUE;
+	gboolean res;
+	
+	gedit_debug (DEBUG_PREFS, "");
+
+	g_return_if_fail (dlg != NULL);
+	g_return_if_fail (selection != NULL);
+
+	selected = gtk_tree_selection_get_selected (selection, NULL, NULL) &&
+		gedit_prefs_manager_encodings_can_set ();
+	
+	gtk_widget_set_sensitive (dlg->priv->remove_enc_button, selected);
+
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (dlg->priv->encodings_treeview));
+	g_return_if_fail (model != NULL);
+
+	n_rows = gtk_tree_model_iter_n_children (model, NULL);
+
+	if (n_rows > 0)
+	{
+		res = gtk_tree_model_get_iter_first (model, &iter);
+		g_return_if_fail (res);
+		disable_up = gtk_tree_selection_iter_is_selected (selection, &iter);
+	
+		res = gtk_tree_model_iter_nth_child (model, &iter, NULL, n_rows - 1);
+		g_return_if_fail (res);
+		disable_down = gtk_tree_selection_iter_is_selected (selection, &iter);
+	}
+	
+	gtk_widget_set_sensitive (dlg->priv->up_enc_button, selected && !disable_up);
+	gtk_widget_set_sensitive (dlg->priv->down_enc_button, selected && !disable_down);
+}
+
+static void
+gedit_preferences_dialog_up_enc_button_clicked (GtkButton *button, GeditPreferencesDialog *dlg)
+{
+	GtkTreeSelection *selection;
+	GtkTreeIter iter;
+	GtkTreeIter prev_iter;
+	GtkTreePath *path;
+	gboolean res;
+
+	GtkTreeModel *model;
+
+	gedit_debug (DEBUG_PREFS, "");
+
+	g_return_if_fail (dlg != NULL);
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dlg->priv->encodings_treeview));
+	g_return_if_fail (selection != NULL);
+
+	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+		return;
+
+	path = gtk_tree_model_get_path (model, &iter);
+
+	res = gtk_tree_path_prev (path);
+	g_return_if_fail (res);
+
+	gtk_tree_model_get_iter (model, &prev_iter, path);
+	
+	gtk_tree_path_free (path);
+
+	gtk_tree_selection_unselect_iter (selection, &iter);
+
+	gtk_list_store_swap (GTK_LIST_STORE (model), &iter, &prev_iter);
+
+	update_encodings_list (dlg);
+
+	gtk_tree_selection_select_iter (selection, &iter);
+}
+
+static void
+gedit_preferences_dialog_down_enc_button_clicked (GtkButton *button, GeditPreferencesDialog *dlg)
+{
+	GtkTreeSelection *selection;
+	GtkTreeIter iter;
+	GtkTreeIter next_iter;
+	gboolean res;
+
+	GtkTreeModel *model;
+
+	gedit_debug (DEBUG_PREFS, "");
+
+	g_return_if_fail (dlg != NULL);
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dlg->priv->encodings_treeview));
+	g_return_if_fail (selection != NULL);
+
+	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+		return;
+
+	next_iter = iter;
+	
+	res = gtk_tree_model_iter_next (model, &next_iter);
+	g_return_if_fail (res);
+
+	gtk_tree_selection_unselect_iter (selection, &iter);
+
+	gtk_list_store_swap (GTK_LIST_STORE (model), &next_iter, &iter);
+
+	update_encodings_list (dlg);
+
+	gtk_tree_selection_select_iter (selection, &iter);
+}
+
+
+static gboolean 
+gedit_preferences_dialog_setup_load_page (GeditPreferencesDialog *dlg, GladeXML *gui)
+{
+	GtkTreeModel *model;
+	GtkTreeViewColumn *column;
+	GtkCellRenderer *cell;
+	GtkTreeSelection *selection;
+
+	gedit_debug (DEBUG_PREFS, "");
+
+	dlg->priv->encodings_treeview = glade_xml_get_widget (gui, "encodings_treeview");
+	dlg->priv->add_enc_button = glade_xml_get_widget (gui, "add_enc_button");
+	dlg->priv->remove_enc_button = glade_xml_get_widget (gui, "remove_enc_button") ;
+	dlg->priv->up_enc_button = glade_xml_get_widget (gui, "up_enc_button");
+	dlg->priv->down_enc_button = glade_xml_get_widget (gui, "down_enc_button");
+
+	g_return_val_if_fail (dlg->priv->encodings_treeview != NULL, FALSE);
+	g_return_val_if_fail (dlg->priv->add_enc_button != NULL, FALSE);
+	g_return_val_if_fail (dlg->priv->remove_enc_button != NULL, FALSE);
+	g_return_val_if_fail (dlg->priv->up_enc_button != NULL, FALSE);
+	g_return_val_if_fail (dlg->priv->down_enc_button != NULL, FALSE);
+
+	gtk_widget_set_sensitive (dlg->priv->add_enc_button, gedit_prefs_manager_encodings_can_set ());
+	gtk_widget_set_sensitive (dlg->priv->remove_enc_button, FALSE);
+	gtk_widget_set_sensitive (dlg->priv->up_enc_button, FALSE);
+	gtk_widget_set_sensitive (dlg->priv->down_enc_button, FALSE);
+	
+
+	g_signal_connect (G_OBJECT (dlg->priv->add_enc_button), "clicked", 
+			  G_CALLBACK (gedit_preferences_dialog_add_enc_button_clicked), 
+			  dlg);
+
+	g_signal_connect (G_OBJECT (dlg->priv->remove_enc_button), "clicked", 
+			  G_CALLBACK (gedit_preferences_dialog_remove_enc_button_clicked), 
+			  dlg);
+
+	g_signal_connect (G_OBJECT (dlg->priv->up_enc_button), "clicked", 
+			  G_CALLBACK (gedit_preferences_dialog_up_enc_button_clicked), 
+			  dlg);
+
+	g_signal_connect (G_OBJECT (dlg->priv->down_enc_button), "clicked", 
+			  G_CALLBACK (gedit_preferences_dialog_down_enc_button_clicked), 
+			  dlg);
+
+	model = create_encodings_treeview_model ();
+	g_return_val_if_fail (model != NULL, FALSE);
+
+	gtk_tree_view_set_model (GTK_TREE_VIEW (dlg->priv->encodings_treeview), model);
+
+	/* Add the encoding column */
+	cell = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes (_("Encodings"), cell, 
+			"text", COLUMN_ENCODING_NAME, NULL);
+	
+	gtk_tree_view_append_column (GTK_TREE_VIEW (dlg->priv->encodings_treeview), column);
+
+	gtk_tree_view_set_search_column (GTK_TREE_VIEW (dlg->priv->encodings_treeview),
+			COLUMN_ENCODING_NAME);
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dlg->priv->encodings_treeview));
+	g_return_val_if_fail (selection != NULL, FALSE);
+
+	if (gedit_prefs_manager_encodings_can_set ())
+		g_signal_connect (G_OBJECT (selection), "changed", 
+				  G_CALLBACK (gedit_preferences_dialog_encodings_treeview_selection_changed), 
+			  	  dlg);
+
+	return TRUE;
+}
+
+static gboolean
+gedit_preferences_dialog_add_encoding (GeditPreferencesDialog *dlg, const GeditEncoding* enc)
+{
+	GtkTreeModel *model;
+
+	gboolean found = FALSE;
+	GtkTreeIter iter;
+	gchar *name;
+	GValue value = {0, };
+
+	gedit_debug (DEBUG_PREFS, "");
+	
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (dlg->priv->encodings_treeview));
+
+	if (gtk_tree_model_get_iter_first (model, &iter))
+	{
+		gtk_tree_model_get_value (model, &iter,
+			    COLUMN_ENCODING_POINTER, &value);
+
+		if (enc == g_value_get_pointer (&value))
+			found = TRUE;
+			
+		g_value_unset (&value);
+
+		while (gtk_tree_model_iter_next (model, &iter) && !found)
+		{
+			gtk_tree_model_get_value (model, &iter,
+				    COLUMN_ENCODING_POINTER, &value);
+
+			if (enc == g_value_get_pointer (&value))
+				found = TRUE;
+			
+				g_value_unset (&value);
+		}
+	}
+
+	if (found)
+		return FALSE;
+
+	name = gedit_encoding_to_string (enc);
+		
+	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+
+	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+				    COLUMN_ENCODING_NAME, name,
+				    COLUMN_ENCODING_POINTER, enc,
+				    COLUMN_INDEX, gtk_tree_model_iter_n_children (model, NULL),
+				    -1);
+	g_free (name);
+
+	return TRUE;
+}
+
+gboolean
+gedit_preferences_dialog_add_encodings (GeditPreferencesDialog *dlg, const GSList* encs)
+{
+	gboolean changed = FALSE;
+
+	while (encs != NULL)
+	{
+		const GeditEncoding* enc = (const GeditEncoding *)encs->data;
+		
+		changed |= gedit_preferences_dialog_add_encoding (dlg, enc);
+
+		encs = g_slist_next (encs);
+	}
+	
+	if (changed)
+		update_encodings_list (dlg);
+
+	return changed;
+}
+
+
