@@ -37,6 +37,7 @@
 #include <libgnomevfs/gnome-vfs.h>
 
 #include <eel/eel-vfs-extensions.h>
+#include <eel/eel-string.h>
 
 #include "gedit2.h"
 #include "gedit-file.h"
@@ -46,7 +47,7 @@
 #include "gedit-recent.h" 
 #include "gedit-file-selector-util.h"
 #include "gedit-plugins-engine.h"
-#include "gnome-recent-model.h"
+#include "recent-files/egg-recent-model.h"
 #include "gedit-prefs-manager.h"
 
 static gchar 	*get_dirname_from_uri 		(const char *uri);
@@ -162,6 +163,14 @@ gedit_file_open (GeditMDIChild *active_child)
 			{
 				gchar *uri_utf8;
 				
+				if (gedit_utils_uri_has_file_scheme (files[i]))
+				{				
+					if (gedit_default_path != NULL)
+						g_free (gedit_default_path);
+
+					gedit_default_path = get_dirname_from_uri (files[i]);
+				}
+				
 				uri_utf8 = eel_format_uri_for_display (files[i]);	
 				if (uri_utf8 != NULL)
 				{
@@ -169,14 +178,7 @@ gedit_file_open (GeditMDIChild *active_child)
 
 					g_free (uri_utf8);
 				}
-
-				if (gedit_utils_uri_has_file_scheme (files[i]))
-				{				
-					if (gedit_default_path != NULL)
-						g_free (gedit_default_path);
-
-					gedit_default_path = get_dirname_from_uri (files[i]);
-				}				
+							
 			}
 			
 			gedit_debug (DEBUG_FILE, "File: %s", files[i]);
@@ -191,25 +193,15 @@ gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 {
 	GError *error = NULL;
 	gchar *uri;
-	gchar *uri_utf8;
-	GnomeRecentModel *recent;
+	
+	EggRecentModel *recent;
 
 	gedit_debug (DEBUG_FILE, "File name: %s", file_name);
 
 	uri = eel_make_uri_canonical (file_name);
 	g_return_val_if_fail (uri != NULL, FALSE);
 	gedit_debug (DEBUG_FILE, "Canonical uri: %s", uri);
-
-	uri_utf8 = g_filename_to_utf8 (uri, -1, NULL, NULL, NULL);
-	if (uri_utf8 == NULL)
-	{
-		g_free (uri);
-
-		return FALSE;
-	}
 	
-	gedit_debug (DEBUG_FILE, "URI: %s", uri);
-
 	if (active_child == NULL ||
 	    !gedit_document_is_untouched (active_child->document))	     
 	{
@@ -255,9 +247,8 @@ gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 	}
 	
 	recent = gedit_recent_get_model ();
-	gnome_recent_model_add (recent, uri_utf8);
+	egg_recent_model_add (recent, uri);
 
-	g_free (uri_utf8);
 	g_free (uri);
 
 	gedit_debug (DEBUG_FILE, "END: %s\n", file_name);
@@ -325,7 +316,14 @@ gedit_file_save (GeditMDIChild* child)
 		view = GTK_WIDGET (g_list_nth_data (
 					bonobo_mdi_child_get_views (BONOBO_MDI_CHILD (child)), 0));
 		if (view != NULL)
+		{
+			GtkWindow *window;
+
+			window = GTK_WINDOW (bonobo_mdi_get_window_from_view (view));
+			gtk_window_present (window);
+
 			bonobo_mdi_set_active_view (BONOBO_MDI (gedit_mdi), view);
+		}
 		
 		gedit_utils_error_reporting_saving_file (uri, error,
 					GTK_WINDOW (gedit_get_active_window ()));
@@ -340,11 +338,9 @@ gedit_file_save (GeditMDIChild* child)
 	}	
 	else
 	{
-		GnomeRecentModel *recent;
+		EggRecentModel *recent;
 		gchar *raw_uri;
-		gchar *canonical_uri;
-		gchar *canonical_uri_utf8;
-
+		
 		gedit_debug (DEBUG_FILE, "OK");
 
 		gedit_utils_flash_va (_("File '%s' saved."), uri);
@@ -354,20 +350,11 @@ gedit_file_save (GeditMDIChild* child)
 		raw_uri = gedit_document_get_raw_uri (doc);
 		g_return_val_if_fail (raw_uri != NULL, TRUE);
 		
-		canonical_uri = eel_make_uri_canonical (raw_uri);
-		g_return_val_if_fail (canonical_uri != NULL, TRUE);
-
-		/* canonical_uri is not valid utf8 */
-		canonical_uri_utf8 = g_filename_to_utf8 (canonical_uri, -1, NULL, NULL, NULL);
-		g_return_val_if_fail (canonical_uri_utf8 != NULL, TRUE);
-
 		recent = gedit_recent_get_model ();
-		gnome_recent_model_add (recent, canonical_uri_utf8);
+		egg_recent_model_add (recent, raw_uri);
 
 		g_free (raw_uri);
-		g_free (canonical_uri);
-		g_free (canonical_uri_utf8);
-
+		
 		return TRUE;
 	}
 }
@@ -380,7 +367,7 @@ gedit_file_save_as (GeditMDIChild *child)
 	GeditDocument *doc;
 	gchar *fname = NULL;
 	gchar *path = NULL;
-	gchar *uri = NULL;
+	gchar *raw_uri = NULL;
 	
 	gedit_debug (DEBUG_FILE, "");
 
@@ -389,21 +376,30 @@ gedit_file_save_as (GeditMDIChild *child)
 	doc = child->document;
 	g_return_val_if_fail (doc != NULL, FALSE);
 
-	uri = gedit_document_get_uri (doc);
-	g_return_val_if_fail (uri != NULL, FALSE);
+	raw_uri = gedit_document_get_raw_uri (doc);
 
 	if (gedit_document_is_untitled (doc))
 	{
+		char *tmpstr;
+		
 		path = (gedit_default_path != NULL) ? 
 			g_strdup (gedit_default_path) : NULL;
-		fname = g_strdup (uri);
+
+		tmpstr = gedit_document_get_uri (doc);
+		fname = g_filename_from_utf8 (tmpstr, -1, NULL, NULL, NULL);
+		if (!fname)
+			/* FIXME: should fname remain NULL in this case? - Paolo */		
+			fname = g_strdup ("Untitled"); /* Use ASCII */
+		g_free (tmpstr);
 	}
 	else
 	{
-		fname = eel_uri_get_basename (uri);
+		g_return_val_if_fail (raw_uri != NULL, FALSE);
 
-		if (gedit_utils_uri_has_file_scheme (uri))
-			path = get_dirname_from_uri (uri);
+		fname = eel_uri_get_basename (raw_uri);
+
+		if (gedit_utils_uri_has_file_scheme (raw_uri))
+			path = get_dirname_from_uri (raw_uri);
 		else
 			path = (gedit_default_path != NULL) ? 
 				g_strdup (gedit_default_path) : NULL;
@@ -419,7 +415,7 @@ gedit_file_save_as (GeditMDIChild *child)
 			path,
 			fname);
 	
-	g_free (uri);
+	g_free (raw_uri);
 	g_free (fname);
 	
 	if (path != NULL)
@@ -427,33 +423,38 @@ gedit_file_save_as (GeditMDIChild *child)
 
 	if (file != NULL) 
 	{
+		gchar *uri;
 		gchar *file_utf8;
 
-		file_utf8 = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+		uri = eel_make_uri_from_shell_arg (file);
+		g_return_val_if_fail (uri != NULL, FALSE);
 
+		file_utf8 = eel_format_uri_for_display (uri);
 		if (file_utf8 != NULL)
 			gedit_utils_flash_va (_("Saving file '%s' ..."), file_utf8);
 		
-		ret = gedit_file_save_as_real (file, child);
+		ret = gedit_file_save_as_real (uri, child);
 		
 		if (ret)
-		{
-			if (file_utf8 != NULL)
-				gedit_utils_flash_va (_("File '%s' saved."), file_utf8);
-
+		{			
 			if (gedit_default_path != NULL)
 				g_free (gedit_default_path);
 
 			gedit_default_path = get_dirname_from_uri (file);
+
+			if (file_utf8 != NULL)
+				gedit_utils_flash_va (_("File '%s' saved."), file_utf8);
 		}
 		else
 			gedit_utils_flash_va (_("The document has not been saved."));
 
 		gedit_debug (DEBUG_FILE, "File: %s", file);
+		g_free (uri);
 		g_free (file);
 
 		if (file_utf8 != NULL)
 			g_free (file_utf8);
+		
 	}
 
 	return ret;
@@ -483,7 +484,7 @@ gedit_file_save_as_real (const gchar* file_name, GeditMDIChild *child)
 	{
 		g_return_val_if_fail (error != NULL, FALSE);
 
-		gedit_utils_error_reporting_saving_file (uri, error,
+		gedit_utils_error_reporting_saving_file (file_name, error,
 					GTK_WINDOW (gedit_get_active_window ()));
 
 		g_error_free (error);
@@ -495,25 +496,11 @@ gedit_file_save_as_real (const gchar* file_name, GeditMDIChild *child)
 	}	
 	else
 	{
-		gchar *temp;
+		EggRecentModel *recent;
 
-		gedit_debug (DEBUG_FILE, "OK");
+		recent = gedit_recent_get_model ();
+		egg_recent_model_add (recent, uri);
 
-		/* uri is not valid utf8 */
-		
-		temp = g_filename_to_utf8 (uri, -1, NULL, NULL, NULL);
-
-		if (temp != NULL)
-		{
-			GnomeRecentModel *recent;
-
-			recent = gedit_recent_get_model ();
-			gnome_recent_model_add (recent, temp);
-
-			g_free (temp);
-		}
-
-		
 		g_free (uri);
 
 		return TRUE;
@@ -541,11 +528,18 @@ gedit_file_close_all (void)
 void
 gedit_file_exit (void)
 {
+	BonoboWindow *active_window;
+	
 	gedit_debug (DEBUG_FILE, "");
 	
 	if (!gedit_file_close_all ())
 		return;
 
+	active_window = gedit_get_active_window ();
+
+	if (active_window != NULL)
+		gedit_prefs_manager_save_window_size_and_state (active_window);
+
 	gedit_debug (DEBUG_FILE, "All files closed.");
 	
 	gedit_plugins_engine_save_settings ();
@@ -590,7 +584,51 @@ gedit_file_save_all (void)
 	}
 
 	if (view !=  bonobo_mdi_get_active_view (BONOBO_MDI (gedit_mdi)))
+	{
+		GtkWindow *window;
+
+		window = GTK_WINDOW (bonobo_mdi_get_window_from_view (view));
+		gtk_window_present (window);
+
 		bonobo_mdi_set_active_view (BONOBO_MDI (gedit_mdi), view);
+	}
+}
+
+/* Displays a confirmation dialog for whether to revert a file. */
+static gboolean
+gedit_file_revert_dialog (const GeditDocument *doc)
+{
+	GtkWidget *msgbox;
+	gint ret;
+	gchar *name;
+
+	name = gedit_document_get_short_name (doc);
+
+	msgbox = gtk_message_dialog_new (GTK_WINDOW (gedit_get_active_window ()),
+					 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_QUESTION,
+					 GTK_BUTTONS_NONE,
+					 _("Do you want to revert to saved the document \"%s\" ?\n\n"
+					   "You will not be able to undo this operation."), 
+					 name);
+	g_free (name);
+
+	/* Add Cancel button */
+	gtk_dialog_add_button (GTK_DIALOG (msgbox), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+
+	/* Add Revert button */
+	gedit_dialog_add_button (GTK_DIALOG (msgbox), 
+			_("_Revert"), GTK_STOCK_REVERT_TO_SAVED, GTK_RESPONSE_YES);
+
+	gtk_dialog_set_default_response	(GTK_DIALOG (msgbox), GTK_RESPONSE_CANCEL);
+
+	gtk_window_set_resizable (GTK_WINDOW (msgbox), FALSE);
+
+	ret = gtk_dialog_run (GTK_DIALOG (msgbox));
+		
+	gtk_widget_destroy (msgbox);
+
+	return (ret == GTK_RESPONSE_YES);
 }
 
 gboolean
@@ -607,10 +645,13 @@ gedit_file_revert (GeditMDIChild *child)
 	
 	doc = child->document;
 	g_return_val_if_fail (doc != NULL, FALSE);
-
+	
+	if (!gedit_file_revert_dialog (doc))
+		return FALSE;
+	
 	uri = gedit_document_get_uri (doc);
 	g_return_val_if_fail (uri != NULL, FALSE);
-
+	
 	gedit_utils_flash_va (_("Reverting file '%s' ..."), uri);	
 	
 	ret = gedit_document_revert (doc, &error);
@@ -642,32 +683,11 @@ gedit_file_revert (GeditMDIChild *child)
 	}
 }
 
-static gchar *
-gedit_file_make_uri_from_shell_arg (const char *location)
-{
-	gchar* uri;
-	gchar* canonical_uri;
-	
-	gedit_debug (DEBUG_FILE, "LOCATION: %s", location);
-	
-	uri = eel_make_uri_from_shell_arg (location);
-
-	gedit_debug (DEBUG_FILE, "URI: %s", uri);
-
-	canonical_uri = gnome_vfs_unescape_string_for_display (uri);
-
-	gedit_debug (DEBUG_FILE, "CANONICAL URI: %s", canonical_uri);
-
-	g_free (uri);
-
-	return canonical_uri;
-}
-
 gboolean 
-gedit_file_open_uri_list (GList* uri_list, gint line)
+gedit_file_open_uri_list (GList* uri_list, gint line, gboolean create)
 {
 	gchar *full_path;
-	gboolean ret = TRUE;
+	gboolean ret = FALSE;
 	gint loaded_files = 0;
 	BonoboMDIChild *active_child = NULL;
 	GeditView* view = NULL;
@@ -681,21 +701,71 @@ gedit_file_open_uri_list (GList* uri_list, gint line)
 	l = g_list_length (uri_list);
 	
 	if (l > 1)
-		gedit_utils_flash_va (_("Loading %d file..."), l);
+		gedit_utils_flash_va (_("Loading %d files..."), l);
 
 	active_child = bonobo_mdi_get_active_child (BONOBO_MDI (gedit_mdi));
 
         /* create a file for each document in the parameter list */
 	for ( ; uri_list; uri_list = g_list_next (uri_list))
 	{
-		full_path = gedit_file_make_uri_from_shell_arg (uri_list->data);
+		full_path = eel_make_uri_from_shell_arg (uri_list->data);
+	
 		if (full_path != NULL) 
 		{
-			if (gedit_file_open_real (full_path, 
-					(active_child != NULL) ? GEDIT_MDI_CHILD (active_child): NULL))
+			if (!gedit_utils_uri_has_file_scheme (full_path) || gedit_utils_uri_exists (full_path))
 			{
-				++loaded_files;
-				ret |= TRUE;
+				if (gedit_file_open_real (full_path, 
+					(active_child != NULL) ? GEDIT_MDI_CHILD (active_child): NULL))
+				{
+					++loaded_files;
+					ret |= TRUE;
+				}
+			}
+			else
+			{
+				if (create)
+				{
+					GtkWidget *dialog;
+					gboolean created = FALSE;
+					gchar *formatted_path;
+
+					formatted_path = eel_format_uri_for_display (full_path);
+					g_return_val_if_fail (formatted_path != NULL, FALSE);
+					
+					dialog = gtk_message_dialog_new (
+						GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))),
+						GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					   	GTK_MESSAGE_QUESTION,
+					   	GTK_BUTTONS_YES_NO,
+						_("The file \"%s\" does not exist. Would you like to create it?"),
+						formatted_path);
+
+					g_free (formatted_path);
+
+					gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_NO);
+
+					gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
+					if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
+					{
+						created = gedit_utils_create_empty_file (full_path);
+
+						if (!created)	
+							gedit_utils_error_reporting_creating_file (full_path, errno, GTK_WINDOW (dialog));
+					}
+									
+					gtk_widget_destroy (dialog);
+
+					if (created)
+					{
+						if (gedit_file_open_real (full_path, 
+							(active_child != NULL) ? GEDIT_MDI_CHILD (active_child): NULL))
+						{
+							++loaded_files;
+							ret |= TRUE;
+						}
+					}
+				}
 			}
 			
 			g_free (full_path);
@@ -708,11 +778,16 @@ gedit_file_open_uri_list (GList* uri_list, gint line)
 	if (view != NULL)
 	{
 		GeditDocument *doc;
+		GtkWindow *window;
 
 		doc = gedit_view_get_document (view);
 		g_return_val_if_fail (doc, FALSE);
 		
 		gedit_document_goto_line (doc, MAX (0, line - 1));
+
+		window = GTK_WINDOW (bonobo_mdi_get_window_from_view (GTK_WIDGET (view)));
+		gtk_window_present (window);
+
 		bonobo_mdi_set_active_view (BONOBO_MDI (gedit_mdi), GTK_WIDGET (view));		
 	}
 
@@ -723,55 +798,47 @@ gedit_file_open_uri_list (GList* uri_list, gint line)
 }
 
 gboolean 
-gedit_file_open_recent (GnomeRecentView *view, const gchar *uri, gpointer data)
+gedit_file_open_recent (EggRecentView *view, EggRecentItem *item, gpointer data)
 {
 	gboolean ret = FALSE;
 	GeditView* active_view;
-	gchar *temp;
+	gchar *uri_utf8;
 
-	gedit_debug (DEBUG_FILE, "Open : %s", uri);
+	uri_utf8 = egg_recent_item_get_uri_utf8 (item);
 
-	temp = g_filename_from_utf8 (uri, -1, NULL, NULL, NULL);
+	gedit_debug (DEBUG_FILE, "Open : %s", uri_utf8);
 
-	if (temp != NULL)
-	{
-		ret = gedit_file_open_single_uri (temp);
+	/* Note that gedit_file_open_single_uri takes a possibly mangled "uri", in UTF8 */
+
+	ret = gedit_file_open_single_uri (uri_utf8);
 	
-		if (ret) {
-			GnomeRecentModel *model;
+	if (!ret) 
+	{
+		EggRecentModel *model;
+		gchar *uri;
 
-			model = gedit_recent_get_model ();
-			gnome_recent_model_add (model, uri);
-		}
+		uri = egg_recent_item_get_uri (item);
+
+		model = gedit_recent_get_model ();
+		egg_recent_model_delete (model, uri);
+
+		g_free (uri);
 	}
 		
 	active_view = gedit_get_active_view ();
 	if (active_view != NULL)
 		gtk_widget_grab_focus (GTK_WIDGET (active_view));
 
+	g_free (uri_utf8);
+
 	gedit_debug (DEBUG_FILE, "END");
 
 	return ret;
 }
 
-
-static gchar *
-gedit_file_make_canonical_uri_from_input (const char *location)
-{
-	gchar* uri;
-	gchar* canonical_uri;
-	
-	gedit_debug (DEBUG_FILE, "");
-	
-	uri = eel_make_uri_from_input (location);
-
-	canonical_uri = eel_make_uri_canonical (uri);
-
-	g_free (uri);
-
-	return canonical_uri;
-}
-
+/*
+ *  uri: a possibly mangled "uri", in UTF8
+ */
 gboolean 
 gedit_file_open_single_uri (const gchar* uri)
 {
@@ -782,7 +849,7 @@ gedit_file_open_single_uri (const gchar* uri)
 	
 	if (uri == NULL) return FALSE;
 	
-	full_path = gedit_file_make_canonical_uri_from_input (uri);
+	full_path = eel_make_uri_from_input (uri);
 
 	if (full_path != NULL) 
 	{
@@ -796,8 +863,8 @@ gedit_file_open_single_uri (const gchar* uri)
 		{
 			gchar *uri_utf8;
 
-			uri_utf8 = g_filename_to_utf8 (full_path, -1, NULL, NULL, NULL);
-
+			uri_utf8 = eel_format_uri_for_display (full_path);
+			
 			if (uri_utf8 != NULL)
 			{
 				gedit_utils_flash_va (_("Loaded file '%s'"), uri_utf8);
