@@ -41,7 +41,7 @@
 #include "gedit-view.h"
 #include "gedit-mdi.h"
 #include "gedit2.h"
-
+#include "gedit-encodings.h"
 
 #define GPM_PREFS_DIR			GEDIT_BASE_KEY "/preferences"
 
@@ -78,6 +78,9 @@
 #define GPM_LINE_NUMBERS_DIR		GPM_PREFS_DIR "/editor/line_numbers"
 #define GPM_DISPLAY_LINE_NUMBERS 	GPM_LINE_NUMBERS_DIR "/display_line_numbers"
 
+#define GPM_LOAD_DIR			GPM_PREFS_DIR "/editor/load"
+#define GPM_ENCODINGS			GPM_LOAD_DIR "/encodings"
+
 /* UI */
 #define GPM_TOOLBAR_DIR			GPM_PREFS_DIR "/ui/toolbar"
 #define GPM_TOOLBAR_VISIBLE	 	GPM_TOOLBAR_DIR "/toolbar_visible"
@@ -110,7 +113,7 @@
 /* Fallback default values. Keep in sync with gedit.schemas */
 
 #define GPM_DEFAULT_USE_DEFAULT_FONT 	0 /* FALSE */
-#define GPM_DEFAULT_EDITOR_FONT 	(const gchar*) "Courier Medium 12"
+#define GPM_DEFAULT_EDITOR_FONT 	(const gchar*) "Monospace 12"
 
 #define GPM_DEFAULT_USE_DEFAULT_COLORS 	1 /* TRUE */
 #define GPM_DEFAULT_BACKGROUND_COLOR	(const gchar*) "#ffffffffffff"
@@ -134,6 +137,8 @@
 
 #define GPM_DEFAULT_DISPLAY_LINE_NUMBERS 0 /* FALSE */
 
+#define GPM_DEFAULT_ENCODINGS		{"ISO-8859-15", NULL}
+       	
 #define GPM_DEFAULT_TOOLBAR_VISIBLE	1 /* TRUE */
 #define GPM_DEFAULT_TOOLBAR_BUTTONS_STYLE "GEDIT_TOOLBAR_SYSTEM"
 #define GPM_DEFAULT_TOOLBAR_SHOW_TOOLTIPS 1 /* TRUE */
@@ -146,9 +151,9 @@
 #define GPM_DEFAULT_PRINT_WRAP_MODE	"GTK_WRAP_CHAR"
 #define GPM_DEFAULT_PRINT_LINE_NUMBERS	0 /* No numbers */
 
-#define GPM_DEFAULT_PRINT_FONT_BODY 	(const gchar*) "Courier 9"
-#define GPM_DEFAULT_PRINT_FONT_HEADER	(const gchar*) "Helvetica 11"
-#define GPM_DEFAULT_PRINT_FONT_NUMBERS	(const gchar*) "Helvetica 8"
+#define GPM_DEFAULT_PRINT_FONT_BODY 	(const gchar*) "Monospace 9"
+#define GPM_DEFAULT_PRINT_FONT_HEADER	(const gchar*) "Sans Normal 11"
+#define GPM_DEFAULT_PRINT_FONT_NUMBERS	(const gchar*) "Sans Normal 8"
 
 #define GPM_DEFAULT_MAX_RECENTS		5
 
@@ -1049,16 +1054,34 @@ DEFINE_STRING_PREF (print_font_body,
 		    GPM_PRINT_FONT_BODY,
 		    GPM_DEFAULT_PRINT_FONT_BODY);
 
+const gchar *
+gedit_prefs_manager_get_default_print_font_body (void)
+{
+	return GPM_DEFAULT_PRINT_FONT_BODY;
+}
+
 /* Font used to print headers */
 DEFINE_STRING_PREF (print_font_header,
 		    GPM_PRINT_FONT_HEADER,
 		    GPM_DEFAULT_PRINT_FONT_HEADER);
 
+const gchar *
+gedit_prefs_manager_get_default_print_font_header (void)
+{
+	return GPM_DEFAULT_PRINT_FONT_HEADER;
+}
+
 /* Font used to print line numbers */
 DEFINE_STRING_PREF (print_font_numbers,
 		    GPM_PRINT_FONT_NUMBERS,
 		    GPM_DEFAULT_PRINT_FONT_NUMBERS);
 
+const gchar *
+gedit_prefs_manager_get_default_print_font_numbers (void)
+{
+	return GPM_DEFAULT_PRINT_FONT_NUMBERS;
+}
+
 
 /* Max number of files in "Recent Files" menu. 
  * This is configurable only using gconftool or gconf-editor 
@@ -1488,6 +1511,121 @@ gedit_prefs_manager_window_width_can_set (void)
 	return TRUE;
 }
 
+void
+gedit_prefs_manager_save_window_size_and_state (BonoboWindow *window)
+{
+	const BonoboMDIWindowInfo *window_info;
+
+	gedit_debug (DEBUG_PREFS, "");
+	
+	g_return_if_fail (window != NULL);
+	g_return_if_fail (BONOBO_IS_WINDOW (window));
+
+	window_info = bonobo_mdi_get_window_info (window);
+	g_return_if_fail (window_info != NULL);
+	
+	if (gedit_prefs_manager_window_height_can_set ())
+		gedit_prefs_manager_set_window_height (window_info->height);
+
+	if (gedit_prefs_manager_window_width_can_set ())
+		gedit_prefs_manager_set_window_width (window_info->width);
+
+	if (gedit_prefs_manager_window_state_can_set ())
+		gedit_prefs_manager_set_window_state (window_info->state);
+}
+
+
+/* Encodings */
+GSList *
+gedit_prefs_manager_get_encodings (void)
+{
+	GSList *strings;
+	GSList *res = NULL;
+	gedit_debug (DEBUG_PREFS, "");
+
+	g_return_val_if_fail (gedit_prefs_manager != NULL, NULL);
+	g_return_val_if_fail (gedit_prefs_manager->gconf_client != NULL, NULL);
+
+	strings = gconf_client_get_list (gedit_prefs_manager->gconf_client,
+				GPM_ENCODINGS,
+				GCONF_VALUE_STRING, 
+				NULL);
+
+	if (strings != NULL)
+	{	
+		GSList *tmp;
+		const GeditEncoding *enc;
+
+		tmp = strings;
+		
+		while (tmp)
+		{
+		      const char *charset = tmp->data;
+
+		      if (strcmp (charset, "current") == 0)
+			      g_get_charset (&charset);
+      
+		      g_return_val_if_fail (charset != NULL, NULL);
+		      enc = gedit_encoding_get_from_charset (charset);
+		      
+		      if (enc != NULL)
+				res = g_slist_prepend (res, (gpointer)enc);
+
+		      tmp = g_slist_next (tmp);
+		}
+
+		g_slist_foreach (strings, (GFunc) g_free, NULL);
+		g_slist_free (strings);    
+
+	 	res = g_slist_reverse (res);
+	}
+	return res;
+}
+
+void
+gedit_prefs_manager_set_encodings (const GSList *encs)
+{	
+	GSList *list = NULL;
+	
+	g_return_if_fail (gedit_prefs_manager != NULL);
+	g_return_if_fail (gedit_prefs_manager->gconf_client != NULL);
+	g_return_if_fail (gedit_prefs_manager_encodings_can_set ());
+
+	while (encs != NULL)
+	{
+		const GeditEncoding *enc;
+		const gchar *charset;
+		
+		enc = (const GeditEncoding *)encs->data;
+
+		charset = gedit_encoding_get_charset (enc);
+		g_return_if_fail (charset != NULL);
+
+		list = g_slist_prepend (list, (gpointer)charset);
+
+		encs = g_slist_next (encs);
+	}
+
+	list = g_slist_reverse (list);
+		
+	gconf_client_set_list (gedit_prefs_manager->gconf_client,
+			GPM_ENCODINGS,
+			GCONF_VALUE_STRING,
+		       	list,
+			NULL);	
+	
+	g_slist_free (list);
+}
+
+gboolean
+gedit_prefs_manager_encodings_can_set (void)
+{
+	gedit_debug (DEBUG_PREFS, "");
+	
+	return gedit_prefs_manager_key_is_writable (GPM_ENCODINGS);
+
+}
+
 
 /* The following functions are taken from gconf-client.c 
  * and partially modified. 
