@@ -222,7 +222,7 @@ gboolean		 gedit_prefs_manager_window_width_can_set	(void);
 void			 gedit_prefs_manager_save_window_size_and_state (BonoboWindow *window);
 
 /* Encodings */
-GSList const		*gedit_prefs_manager_get_encodings		(void);
+GSList 			*gedit_prefs_manager_get_encodings		(void);
 void			 gedit_prefs_manager_set_encodings		(const GSList *encs);
 gboolean 		 gedit_prefs_manager_encodings_can_set		(void);
 
