@@ -33,13 +33,13 @@
 #endif
 
 #include "gedit-recent.h"
-#include "gedit-prefs.h"
+#include "gedit-prefs-manager.h"
 
 
-static GnomeRecentModel *model;
+static EggRecentModel *model;
 
 
-GnomeRecentModel *
+EggRecentModel *
 gedit_recent_get_model (void)
 {
 	return model;
@@ -48,5 +48,8 @@ gedit_recent_get_model (void)
 void
 gedit_recent_init (void)
 {
-	model = gnome_recent_model_new ("gedit-2", gedit_settings->max_recents);
+	model = egg_recent_model_new (EGG_RECENT_MODEL_SORT_MRU);
+	egg_recent_model_set_limit (model,
+				gedit_prefs_manager_get_max_recents ());
+	egg_recent_model_set_filter_mime_types (model, "text/*", NULL);
 }
